@@ -2,6 +2,7 @@ import csv
 import io
 import os
 from pathlib import Path
+from time import sleep
 from typing import IO, Any, List, Union
 from urllib.parse import urlparse
 
@@ -63,7 +64,18 @@ class CSVUrlReader(Reader):
             raise ImportError("`httpx` not installed")
 
         logger.info(f"Reading: {url}")
-        response = httpx.get(url)
+        # Retry the request up to 3 times with exponential backoff
+        for attempt in range(3):
+            try:
+                response = httpx.get(url)
+                break
+            except httpx.RequestError as e:
+                if attempt == 2:  # Last attempt
+                    logger.error(f"Failed to fetch CSV after 3 attempts: {e}")
+                    raise
+                wait_time = 2**attempt  # Exponential backoff: 1, 2, 4 seconds
+                logger.warning(f"Request failed, retrying in {wait_time} seconds...")
+                sleep(wait_time)
 
         try:
             response.raise_for_status()
