@@ -15,11 +15,11 @@ except ImportError:
 class FastEmbedEmbedder(Embedder):
     """Using BAAI/bge-small-en-v1.5 model, more models available: https://qdrant.github.io/fastembed/examples/Supported_Models/"""
 
-    model: str = "BAAI/bge-small-en-v1.5"
+    id: str = "BAAI/bge-small-en-v1.5"
     dimensions: int = 384
 
     def get_embedding(self, text: str) -> List[float]:
-        model = TextEmbedding(model_name=self.model)
+        model = TextEmbedding(model_name=self.id)
         embeddings = model.embed(text)
         embedding_list = list(embeddings)
 
