@@ -5,7 +5,7 @@ from os import getenv
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 from agno.exceptions import ModelProviderError, ModelRateLimitError
-from agno.media import Image
+from agno.media import File, Image
 from agno.models.base import Model
 from agno.models.message import Message
 from agno.models.response import ModelResponse
@@ -17,14 +17,11 @@ try:
     from anthropic import AsyncAnthropic as AsyncAnthropicClient
     from anthropic.types import (
         ContentBlockDeltaEvent,
+        ContentBlockStartEvent,
         ContentBlockStopEvent,
         MessageDeltaEvent,
         MessageStopEvent,
-        RedactedThinkingBlock,
         TextBlock,
-        TextDelta,
-        ThinkingBlock,
-        ThinkingDelta,
         ToolUseBlock,
     )
     from anthropic.types import Message as AnthropicMessage
@@ -51,7 +48,7 @@ def _format_image_for_message(image: Image) -> Optional[Dict[str, Any]]:
     try:
         # Case 1: Image is a URL
         if image.url is not None:
-            content_bytes = image.image_url_content
+            return {"type": "image", "source": {"type": "url", "url": image.url}}
 
         # Case 2: Image is a local file path
         elif image.filepath is not None:
@@ -97,6 +94,54 @@ def _format_image_for_message(image: Image) -> Optional[Dict[str, Any]]:
         return None
 
 
+def _format_file_for_message(file: File) -> Optional[Dict[str, Any]]:
+    """
+    Add a document url or base64 encoded content to a message.
+    """
+    # Case 1: Document is a URL
+    if file.url is not None:
+        return {
+            "type": "document",
+            "source": {
+                "type": "url",
+                "url": file.url,
+            },
+        }
+    # Case 2: Document is a local file path
+    elif file.filepath is not None:
+        import base64
+        from pathlib import Path
+
+        path = Path(file.filepath) if isinstance(file.filepath, str) else file.filepath
+        if path.exists() and path.is_file():
+            file_data = base64.standard_b64encode(path.read_bytes()).decode("utf-8")
+            media_type = file.mime_type
+            if media_type is None:
+                import mimetypes
+
+                media_type = mimetypes.guess_type(file.filepath)[0]
+                if media_type != "application/pdf":
+                    logger.error(f"Unsupported file type: {media_type}")
+            return {
+                "type": "document",
+                "source": {
+                    "type": "base64",
+                    "media_type": media_type,
+                    "data": file_data,
+                },
+            }
+        else:
+            logger.error(f"Document file not found: {file}")
+            return None
+    # Case 3: Document is base64 encoded content
+    elif file.content is not None:
+        return {
+            "type": "document",
+            "source": {"type": "base64", "media_type": "application/pdf", "data": file.content},
+        }
+    return None
+
+
 def _format_messages(messages: List[Message]) -> Tuple[List[Dict[str, str]], str]:
     """
     Process the list of messages and separate them into API messages and system messages.
@@ -110,8 +155,7 @@ def _format_messages(messages: List[Message]) -> Tuple[List[Dict[str, str]], str
     chat_messages: List[Dict[str, str]] = []
     system_messages: List[str] = []
 
-    print()
-    for idx, message in enumerate(messages):
+    for message in messages:
         content = message.content or ""
         if message.role == "system":
             if content is not None:
@@ -127,11 +171,35 @@ def _format_messages(messages: List[Message]) -> Tuple[List[Dict[str, str]], str
                     if image_content:
                         content.append(image_content)
 
+            if message.files is not None:
+                for file in message.files:
+                    file_content = _format_file_for_message(file)
+                    if file_content:
+                        content.append(file_content)
+
         # Handle tool calls from history
         elif message.role == "assistant":
             content = []
+
+            if message.thinking is not None and message.provider_data is not None:
+                from anthropic.types import RedactedThinkingBlock, ThinkingBlock
+
+                content.append(
+                    ThinkingBlock(
+                        thinking=message.thinking,
+                        signature=message.provider_data.get("signature"),
+                        type="thinking",
+                    )
+                )
+
+            if message.redacted_thinking is not None:
+                from anthropic.types import RedactedThinkingBlock
+
+                content.append(RedactedThinkingBlock(data=message.redacted_thinking, type="redacted_thinking"))
+
             if isinstance(message.content, str) and message.content:
                 content.append(TextBlock(text=message.content, type="text"))
+
             if message.tool_calls:
                 for tool_call in message.tool_calls:
                     content.append(
@@ -497,17 +565,20 @@ class Claude(Model):
 
         if response.content:
             for block in response.content:
-                if isinstance(block, TextBlock):
+                if block.type == "text":
                     model_response.content = block.text
-                elif isinstance(block, ThinkingBlock):
+                elif block.type == "thinking":
                     model_response.thinking = block.thinking
-                elif isinstance(block, RedactedThinkingBlock):
-                    model_response.thinking = block.data
+                    model_response.provider_data = {
+                        "signature": block.signature,
+                    }
+                elif block.type == "redacted_thinking":
+                    model_response.redacted_thinking = block.data
 
         # -*- Extract tool calls from the response
         if response.stop_reason == "tool_use":
             for block in response.content:
-                if isinstance(block, ToolUseBlock):
+                if block.type == "tool_use":
                     tool_name = block.name
                     tool_input = block.input
 
@@ -545,17 +616,25 @@ class Claude(Model):
         """
         model_response = ModelResponse()
 
+        if isinstance(response, ContentBlockStartEvent):
+            if response.content_block.type == "redacted_thinking":
+                model_response.redacted_thinking = response.content_block.data
+
         if isinstance(response, ContentBlockDeltaEvent):
             # Handle text content
-            if isinstance(response.delta, TextDelta):
+            if response.delta.type == "text_delta":
                 model_response.content = response.delta.text
             # Handle thinking content
-            if isinstance(response.delta, ThinkingDelta):
+            elif response.delta.type == "thinking_delta":
                 model_response.thinking = response.delta.thinking
+            elif response.delta.type == "signature_delta":
+                model_response.provider_data = {
+                    "signature": response.delta.signature,
+                }
 
         elif isinstance(response, ContentBlockStopEvent):
             # Handle tool calls
-            if isinstance(response.content_block, ToolUseBlock):  # type: ignore
+            if response.content_block.type == "tool_use":  # type: ignore
                 tool_use = response.content_block  # type: ignore
                 tool_name = tool_use.name
                 tool_input = tool_use.input
