@@ -21,10 +21,14 @@ class MessageData:
     response_role: Optional[Literal["system", "user", "assistant", "tool"]] = None
     response_content: Any = ""
     response_thinking: Any = ""
+    response_redacted_thinking: Any = ""
     response_tool_calls: List[Dict[str, Any]] = field(default_factory=list)
     response_audio: Optional[AudioResponse] = None
 
-    extra: Dict[str, Any] = field(default_factory=dict)
+    # Data from the provider that we might need on subsequent messages
+    response_provider_data: Optional[Dict[str, Any]] = None
+
+    extra: Optional[Dict[str, Any]] = None
 
 
 @dataclass
@@ -157,6 +161,7 @@ class Model(ABC):
             ModelResponse: The model's response
         """
         logger.debug(f"---------- {self.get_provider()} Response Start ----------")
+        logger.debug(f"---------- Model: {self.id} ----------")
         self._log_messages(messages)
         model_response = ModelResponse()
 
@@ -219,6 +224,7 @@ class Model(ABC):
             ModelResponse: The model's response
         """
         logger.debug(f"---------- {self.get_provider()} Async Response Start ----------")
+        logger.debug(f"---------- Model: {self.id} ----------")
         self._log_messages(messages)
         model_response = ModelResponse()
 
@@ -311,6 +317,10 @@ class Model(ABC):
                 model_response.content = assistant_message.get_content_string()
             else:
                 model_response.content += assistant_message.get_content_string()
+        if assistant_message.thinking is not None:
+            model_response.thinking = assistant_message.thinking
+        if assistant_message.redacted_thinking is not None:
+            model_response.redacted_thinking = assistant_message.redacted_thinking
         if assistant_message.audio_output is not None:
             model_response.audio = assistant_message.audio_output
         if provider_response.extra is not None:
@@ -361,6 +371,10 @@ class Model(ABC):
                 model_response.content = assistant_message.get_content_string()
             else:
                 model_response.content += assistant_message.get_content_string()
+        if assistant_message.thinking is not None:
+            model_response.thinking = assistant_message.thinking
+        if assistant_message.redacted_thinking is not None:
+            model_response.redacted_thinking = assistant_message.redacted_thinking
         if assistant_message.audio_output is not None:
             model_response.audio = assistant_message.audio_output
         if provider_response.extra is not None:
@@ -405,10 +419,18 @@ class Model(ABC):
         if provider_response.thinking is not None:
             assistant_message.thinking = provider_response.thinking
 
+        # Add redacted thinking content to assistant message
+        if provider_response.redacted_thinking is not None:
+            assistant_message.redacted_thinking = provider_response.redacted_thinking
+
         # Add reasoning content to assistant message
         if provider_response.reasoning_content is not None:
             assistant_message.reasoning_content = provider_response.reasoning_content
 
+        # Add provider data to assistant message
+        if provider_response.provider_data is not None:
+            assistant_message.provider_data = provider_response.provider_data
+
         # Add usage metrics if provided
         if provider_response.response_usage is not None:
             self._add_usage_metrics_to_assistant_message(
@@ -440,6 +462,7 @@ class Model(ABC):
             Iterator[ModelResponse]: Iterator of model responses
         """
         logger.debug(f"---------- {self.get_provider()} Response Stream Start ----------")
+        logger.debug(f"---------- Model: {self.id} ----------")
         self._log_messages(messages)
 
         while True:
@@ -459,6 +482,10 @@ class Model(ABC):
                 assistant_message.content = stream_data.response_content
             if stream_data.response_thinking:
                 assistant_message.thinking = stream_data.response_thinking
+            if stream_data.response_redacted_thinking:
+                assistant_message.redacted_thinking = stream_data.response_redacted_thinking
+            if stream_data.response_provider_data:
+                assistant_message.provider_data = stream_data.response_provider_data
             if stream_data.response_audio:
                 assistant_message.audio_output = stream_data.response_audio
             if stream_data.response_tool_calls and len(stream_data.response_tool_calls) > 0:
@@ -485,9 +512,12 @@ class Model(ABC):
                     yield function_call_response
 
                 # Format and add results to messages
-                self.format_function_call_results(
-                    messages=messages, function_call_results=function_call_results, **stream_data.extra
-                )
+                if stream_data.extra is not None:
+                    self.format_function_call_results(
+                        messages=messages, function_call_results=function_call_results, **stream_data.extra
+                    )
+                else:
+                    self.format_function_call_results(messages=messages, function_call_results=function_call_results)
 
                 logger.debug(f"---------- {self.get_provider()} Response Stream ----------")
                 self._log_messages(messages)
@@ -528,6 +558,7 @@ class Model(ABC):
             AsyncIterator[ModelResponse]: Async iterator of model responses
         """
         logger.debug(f"---------- {self.get_provider()} Async Response Stream Start ----------")
+        logger.debug(f"---------- Model: {self.id} ----------")
         self._log_messages(messages)
 
         while True:
@@ -546,6 +577,12 @@ class Model(ABC):
             # Populate assistant message from stream data
             if stream_data.response_content:
                 assistant_message.content = stream_data.response_content
+            if stream_data.response_thinking:
+                assistant_message.thinking = stream_data.response_thinking
+            if stream_data.response_redacted_thinking:
+                assistant_message.redacted_thinking = stream_data.response_redacted_thinking
+            if stream_data.response_provider_data:
+                assistant_message.provider_data = stream_data.response_provider_data
             if stream_data.response_audio:
                 assistant_message.audio_output = stream_data.response_audio
             if stream_data.response_tool_calls and len(stream_data.response_tool_calls) > 0:
@@ -573,9 +610,12 @@ class Model(ABC):
                     yield function_call_response
 
                 # Format and add results to messages
-                self.format_function_call_results(
-                    messages=messages, function_call_results=function_call_results, **stream_data.extra
-                )
+                if stream_data.extra is not None:
+                    self.format_function_call_results(
+                        messages=messages, function_call_results=function_call_results, **stream_data.extra
+                    )
+                else:
+                    self.format_function_call_results(messages=messages, function_call_results=function_call_results)
 
                 logger.debug(f"---------- {self.get_provider()} Async Response Stream ----------")
                 self._log_messages(messages)
@@ -601,6 +641,10 @@ class Model(ABC):
         if not assistant_message.metrics.time_to_first_token:
             assistant_message.metrics.set_time_to_first_token()
 
+        # Add role to assistant message
+        if model_response.role is not None:
+            assistant_message.role = model_response.role
+
         should_yield = False
         # Update stream_data content
         if model_response.content is not None:
@@ -611,6 +655,15 @@ class Model(ABC):
             stream_data.response_thinking += model_response.thinking
             should_yield = True
 
+        if model_response.redacted_thinking is not None:
+            stream_data.response_redacted_thinking += model_response.redacted_thinking
+            should_yield = True
+
+        if model_response.provider_data:
+            if stream_data.response_provider_data is None:
+                stream_data.response_provider_data = {}
+            stream_data.response_provider_data.update(model_response.provider_data)
+
         # Update stream_data tool calls
         if model_response.tool_calls is not None:
             if stream_data.response_tool_calls is None:
@@ -639,6 +692,8 @@ class Model(ABC):
             should_yield = True
 
         if model_response.extra is not None:
+            if stream_data.extra is None:
+                stream_data.extra = {}
             stream_data.extra.update(model_response.extra)
 
         if model_response.response_usage is not None:
@@ -965,7 +1020,7 @@ class Model(ABC):
             model_response.tool_calls = []
 
         function_calls_to_run: List[FunctionCall] = self.get_function_calls_to_run(assistant_message, messages)
-        if self.show_tool_calls:
+        if self.show_tool_calls and function_calls_to_run:
             self._show_tool_calls(function_calls_to_run, model_response)
         return function_calls_to_run
 
@@ -986,42 +1041,44 @@ class Model(ABC):
             assistant_message: Message to update with metrics
             response_usage: Usage data from model provider
         """
+
         # Standard token metrics
-        if hasattr(response_usage, "input_tokens") and response_usage.input_tokens:
-            assistant_message.metrics.input_tokens = response_usage.input_tokens
-        if hasattr(response_usage, "output_tokens") and response_usage.output_tokens:
-            assistant_message.metrics.output_tokens = response_usage.output_tokens
-        if hasattr(response_usage, "prompt_tokens") and response_usage.prompt_tokens is not None:
-            assistant_message.metrics.input_tokens = response_usage.prompt_tokens
-            assistant_message.metrics.prompt_tokens = response_usage.prompt_tokens
-        if hasattr(response_usage, "completion_tokens") and response_usage.completion_tokens is not None:
-            assistant_message.metrics.output_tokens = response_usage.completion_tokens
-            assistant_message.metrics.completion_tokens = response_usage.completion_tokens
-        if hasattr(response_usage, "total_tokens") and response_usage.total_tokens is not None:
-            assistant_message.metrics.total_tokens = response_usage.total_tokens
+        if isinstance(response_usage, dict):
+            if "input_tokens" in response_usage:
+                assistant_message.metrics.input_tokens = response_usage.get("input_tokens", 0)
+            if "output_tokens" in response_usage:
+                assistant_message.metrics.output_tokens = response_usage.get("output_tokens", 0)
+            if "prompt_tokens" in response_usage:
+                assistant_message.metrics.input_tokens = response_usage.get("prompt_tokens", 0)
+            if "completion_tokens" in response_usage:
+                assistant_message.metrics.output_tokens = response_usage.get("completion_tokens", 0)
+            if "total_tokens" in response_usage:
+                assistant_message.metrics.total_tokens = response_usage.get("total_tokens", 0)
+            else:
+                assistant_message.metrics.total_tokens = (
+                    assistant_message.metrics.input_tokens + assistant_message.metrics.output_tokens
+                )
         else:
-            assistant_message.metrics.total_tokens = (
-                assistant_message.metrics.input_tokens + assistant_message.metrics.output_tokens
-            )
+            if hasattr(response_usage, "input_tokens") and response_usage.input_tokens:
+                assistant_message.metrics.input_tokens = response_usage.input_tokens
+            if hasattr(response_usage, "output_tokens") and response_usage.output_tokens:
+                assistant_message.metrics.output_tokens = response_usage.output_tokens
+            if hasattr(response_usage, "prompt_tokens") and response_usage.prompt_tokens is not None:
+                assistant_message.metrics.input_tokens = response_usage.prompt_tokens
+                assistant_message.metrics.prompt_tokens = response_usage.prompt_tokens
+            if hasattr(response_usage, "completion_tokens") and response_usage.completion_tokens is not None:
+                assistant_message.metrics.output_tokens = response_usage.completion_tokens
+                assistant_message.metrics.completion_tokens = response_usage.completion_tokens
+            if hasattr(response_usage, "total_tokens") and response_usage.total_tokens is not None:
+                assistant_message.metrics.total_tokens = response_usage.total_tokens
+            else:
+                assistant_message.metrics.total_tokens = (
+                    assistant_message.metrics.input_tokens + assistant_message.metrics.output_tokens
+                )
 
-        # Additional timing metrics (e.g., from Groq, Ollama)
-        if assistant_message.metrics.additional_metrics is None:
-            assistant_message.metrics.additional_metrics = {}
-
-        additional_metrics = [
-            "prompt_time",
-            "completion_time",
-            "queue_time",
-            "total_time",
-            "total_duration",
-            "load_duration",
-            "prompt_eval_duration",
-            "eval_duration",
-        ]
-
-        for metric in additional_metrics:
-            if hasattr(response_usage, metric) and getattr(response_usage, metric) is not None:
-                assistant_message.metrics.additional_metrics[metric] = getattr(response_usage, metric)
+        # Additional metrics (e.g., from Groq, Ollama)
+        if isinstance(response_usage, dict) and "additional_metrics" in response_usage:
+            assistant_message.metrics.additional_metrics = response_usage["additional_metrics"]
 
         # Token details (e.g., from OpenAI)
         if hasattr(response_usage, "prompt_tokens_details"):
