@@ -9,9 +9,9 @@ from typing import Any, Dict, List, Optional, Union
 from pydantic import BaseModel
 
 from agno.exceptions import ModelProviderError
-from agno.media import Audio, Image, Video
+from agno.media import Audio, File, Image, Video
 from agno.models.base import Model
-from agno.models.message import Message
+from agno.models.message import Message, MessageMetrics
 from agno.models.response import ModelResponse
 from agno.utils.log import logger
 
@@ -22,7 +22,6 @@ try:
     from google.genai.types import (
         Content,
         DynamicRetrievalConfig,
-        File,
         FunctionDeclaration,
         GenerateContentConfig,
         GenerateContentResponse,
@@ -33,6 +32,9 @@ try:
         Schema,
         Tool,
     )
+    from google.genai.types import (
+        File as GeminiFile,
+    )
 except ImportError:
     raise ImportError("`google-genai` not installed. Please install it using `pip install google-genai`")
 
@@ -40,18 +42,22 @@ except ImportError:
 def _format_image_for_message(image: Image) -> Optional[Dict[str, Any]]:
     # Case 1: Image is a URL
     # Download the image from the URL and add it as base64 encoded data
-    if image.url is not None and image.image_url_content is not None:
-        try:
-            import base64
+    if image.url is not None:
+        content_bytes = image.image_url_content
+        if content_bytes is not None:
+            try:
+                import base64
 
-            content_bytes = image.image_url_content
-            image_data = {
-                "mime_type": "image/jpeg",
-                "data": base64.b64encode(content_bytes).decode("utf-8"),
-            }
-            return image_data
-        except Exception as e:
-            logger.warning(f"Failed to download image from {image}: {e}")
+                image_data = {
+                    "mime_type": "image/jpeg",
+                    "data": base64.b64encode(content_bytes).decode("utf-8"),
+                }
+                return image_data
+            except Exception as e:
+                logger.warning(f"Failed to download image from {image}: {e}")
+                return None
+        else:
+            logger.warning(f"Unsupported image format: {image}")
             return None
 
     # Case 2: Image is a local path
@@ -147,13 +153,6 @@ def _format_function_definitions(tools_list):
         return None
 
 
-@dataclass
-class GeminiResponseUsage:
-    input_tokens: int = 0
-    output_tokens: int = 0
-    total_tokens: int = 0
-
-
 @dataclass
 class Gemini(Model):
     """
@@ -174,8 +173,6 @@ class Gemini(Model):
 
     supports_structured_outputs: bool = True
 
-    assistant_message_role: str = "model"
-
     # Request parameters
     function_declarations: Optional[List[Any]] = None
     generation_config: Optional[Any] = None
@@ -208,6 +205,14 @@ class Gemini(Model):
     # Gemini client
     client: Optional[GeminiClient] = None
 
+    # The role to map the message role to.
+    role_map = {
+        "system": "system",
+        "user": "user",
+        "model": "assistant",
+        "tool": "tool",
+    }
+
     def get_client(self) -> GeminiClient:
         """
         Returns an instance of the GeminiClient client.
@@ -437,13 +442,15 @@ class Gemini(Model):
                 system_message = message.content
                 continue
 
+            role = "model" if role == "assistant" else role
+
             # Add content to the message for the model
             content = message.content
             # Initialize message_parts to be used for Gemini
             message_parts: List[Any] = []
 
             # Function calls
-            if (not content or message.role == "model") and message.tool_calls:
+            if (not content or role == "model") and message.tool_calls:
                 for tool_call in message.tool_calls:
                     message_parts.append(
                         Part.from_function_call(
@@ -452,7 +459,7 @@ class Gemini(Model):
                         )
                     )
             # Function results
-            elif message.role == "tool" and message.tool_calls:
+            elif role == "tool" and message.tool_calls:
                 for tool_call in message.tool_calls:
                     message_parts.append(
                         Part.from_function_response(
@@ -467,7 +474,7 @@ class Gemini(Model):
                 # Add images to the message for the model
                 if message.images is not None:
                     for image in message.images:
-                        if image.content is not None and isinstance(image.content, File):
+                        if image.content is not None and isinstance(image.content, GeminiFile):
                             # Google recommends that if using a single image, place the text prompt after the image.
                             message_parts.insert(0, image.content)
                         else:
@@ -481,7 +488,7 @@ class Gemini(Model):
                         for video in message.videos:
                             # Case 1: Video is a file_types.File object (Recommended)
                             # Add it as a File object
-                            if video.content is not None and isinstance(video.content, File):
+                            if video.content is not None and isinstance(video.content, GeminiFile):
                                 # Google recommends that if using a single image, place the text prompt after the image.
                                 message_parts.insert(
                                     0, Part.from_uri(file_uri=video.content.uri, mime_type=video.content.mime_type)
@@ -501,7 +508,7 @@ class Gemini(Model):
                 if message.audio is not None:
                     try:
                         for audio_snippet in message.audio:
-                            if audio_snippet.content is not None and isinstance(audio_snippet.content, File):
+                            if audio_snippet.content is not None and isinstance(audio_snippet.content, GeminiFile):
                                 # Google recommends that if using a single image, place the text prompt after the image.
                                 message_parts.insert(
                                     0,
@@ -517,11 +524,18 @@ class Gemini(Model):
                         logger.warning(f"Failed to load audio from {message.audio}: {e}")
                         continue
 
+                # Add files to the message for the model
+                if message.files is not None:
+                    for file in message.files:
+                        file_content = self._format_file_for_message(file)
+                        if file_content:
+                            message_parts.append(file_content)
+
             final_message = Content(role=role, parts=message_parts)
             formatted_messages.append(final_message)
         return formatted_messages, system_message
 
-    def _format_audio_for_message(self, audio: Audio) -> Optional[Union[Part, File]]:
+    def _format_audio_for_message(self, audio: Audio) -> Optional[Union[Part, GeminiFile]]:
         # Case 1: Audio is a bytes object
         if audio.content and isinstance(audio.content, bytes):
             return Part.from_bytes(
@@ -578,7 +592,7 @@ class Gemini(Model):
             logger.warning(f"Unknown audio type: {type(audio.content)}")
             return None
 
-    def _format_video_for_message(self, video: Video) -> Optional[File]:
+    def _format_video_for_message(self, video: Video) -> Optional[GeminiFile]:
         # Case 1: Video is a bytes object
         if video.content and isinstance(video.content, bytes):
             return Part.from_bytes(
@@ -629,6 +643,50 @@ class Gemini(Model):
             logger.warning(f"Unknown video type: {type(video.content)}")
             return None
 
+    def _format_file_for_message(self, file: File) -> Optional[Part]:
+        # Case 1: File is a bytes object
+        if file.content and isinstance(file.content, bytes):
+            return Part.from_bytes(mime_type=file.mime_type, data=file.content)
+
+        # Case 2: File is a URL
+        elif file.url is not None:
+            url_content = file.file_url_content
+            if url_content is not None:
+                content, mime_type = url_content
+                return Part.from_bytes(mime_type=mime_type, data=content)
+            else:
+                logger.warning(f"Failed to download file from {file.url}")
+                return None
+
+        # Case 3: File is a local file path
+        elif file.filepath is not None:
+            file_path = file.filepath if isinstance(file.filepath, Path) else Path(file.filepath)
+            if file_path.exists() and file_path.is_file():
+                if file_path.stat().st_size < 20 * 1024 * 1024:  # 20MB in bytes
+                    if file.mime_type is not None:
+                        return Part.from_bytes(mime_type=file.mime_type, data=file_path.read_bytes())
+                    else:
+                        import mimetypes
+
+                        return Part.from_bytes(
+                            mime_type=mimetypes.guess_type(file_path)[0], data=file_path.read_bytes()
+                        )
+                else:
+                    file_upload = self.get_client().files.upload(
+                        file=file_path,
+                    )
+                    # Check whether the file is ready to be used.
+                    while file_upload.state.name == "PROCESSING":
+                        time.sleep(2)
+                        file_upload = self.get_client().files.get(name=file_upload.name)
+                    if file_upload.state.name == "FAILED":
+                        raise ValueError(file_upload.state.name)
+                    return Part.from_uri(file_uri=file_upload.uri, mime_type=file_upload.mime_type)
+            else:
+                logger.error(f"File {file_path} does not exist.")
+
+        return None
+
     def format_function_call_results(
         self, messages: List[Message], function_call_results: List[Message], **kwargs
     ) -> None:
@@ -637,12 +695,19 @@ class Gemini(Model):
         """
         combined_content: List = []
         combined_function_result: List = []
+        message_metrics = MessageMetrics()
         if len(function_call_results) > 0:
             for result in function_call_results:
                 combined_content.append(result.content)
                 combined_function_result.append({"tool_name": result.tool_name, "content": result.content})
+                message_metrics += result.metrics
 
-        messages.append(Message(role="tool", content=combined_content, tool_calls=combined_function_result))
+        if combined_content:
+            messages.append(
+                Message(
+                    role="tool", content=combined_content, tool_calls=combined_function_result, metrics=message_metrics
+                )
+            )
 
     def parse_provider_response(self, response: GenerateContentResponse) -> ModelResponse:
         """
@@ -662,7 +727,7 @@ class Gemini(Model):
 
             # Add role
             if response_message.role is not None:
-                model_response.role = response_message.role
+                model_response.role = self.role_map[response_message.role]
 
             # Add content
             if response_message.parts is not None:
@@ -688,11 +753,11 @@ class Gemini(Model):
         # Extract usage metadata if present
         if hasattr(response, "usage_metadata") and response.usage_metadata is not None:
             usage: GenerateContentResponseUsageMetadata = response.usage_metadata
-            model_response.response_usage = GeminiResponseUsage(
-                input_tokens=usage.prompt_token_count or 0,
-                output_tokens=usage.candidates_token_count or 0,
-                total_tokens=usage.total_token_count or 0,
-            )
+            model_response.response_usage = {
+                "input_tokens": usage.prompt_token_count or 0,
+                "output_tokens": usage.candidates_token_count or 0,
+                "total_tokens": usage.total_token_count or 0,
+            }
 
         return model_response
 
@@ -701,6 +766,10 @@ class Gemini(Model):
 
         response_message: Content = response_delta.candidates[0].content
 
+        # Add role
+        if response_message.role is not None:
+            model_response.role = self.role_map[response_message.role]
+
         if response_message.parts is not None:
             for part in response_message.parts:
                 # Extract text if present
@@ -724,10 +793,10 @@ class Gemini(Model):
         # Extract usage metadata if present
         if hasattr(response_delta, "usage_metadata") and response_delta.usage_metadata is not None:
             usage: GenerateContentResponseUsageMetadata = response_delta.usage_metadata
-            model_response.response_usage = GeminiResponseUsage(
-                input_tokens=usage.prompt_token_count or 0,
-                output_tokens=usage.candidates_token_count or 0,
-                total_tokens=usage.total_token_count or 0,
-            )
+            model_response.response_usage = {
+                "input_tokens": usage.prompt_token_count or 0,
+                "output_tokens": usage.candidates_token_count or 0,
+                "total_tokens": usage.total_token_count or 0,
+            }
 
         return model_response
