@@ -12,7 +12,7 @@ from agno.models.base import Model
 from agno.models.message import Message
 from agno.models.response import ModelResponse
 from agno.utils.log import logger
-from agno.utils.openai import add_audio_to_message, add_images_to_message
+from agno.utils.openai import audio_to_message, images_to_message
 
 try:
     from openai import APIConnectionError, APIStatusError, RateLimitError
@@ -53,8 +53,10 @@ class OpenAIChat(Model):
     top_logprobs: Optional[int] = None
     max_tokens: Optional[int] = None
     max_completion_tokens: Optional[int] = None
-    modalities: Optional[List[str]] = None
-    audio: Optional[Dict[str, Any]] = None
+    modalities: Optional[List[str]] = None  # "text" and/or "audio"
+    audio: Optional[Dict[str, Any]] = (
+        None  # E.g. {"voice": "alloy", "format": "wav"}. `format` must be one of `wav`, `mp3`, `flac`, `opus`, or `pcm16`. `voice` must be one of `ash`, `ballad`, `coral`, `sage`, `verse`, `alloy`, `echo`, and `shimmer`.
+    )
     presence_penalty: Optional[float] = None
     response_format: Optional[Any] = None
     seed: Optional[int] = None
@@ -249,22 +251,44 @@ class OpenAIChat(Model):
         Returns:
             Dict[str, Any]: The formatted message.
         """
-        if message.role == "user":
-            if message.images is not None:
-                message = add_images_to_message(message=message, images=message.images)
+        message_dict: Dict[str, Any] = {
+            "role": self.role_map[message.role],
+            "content": message.content,
+            "name": message.name,
+            "tool_call_id": message.tool_call_id,
+            "tool_calls": message.tool_calls,
+        }
+        message_dict = {k: v for k, v in message_dict.items() if v is not None}
 
-            if message.audio is not None:
-                message = add_audio_to_message(message=message, audio=message.audio)
+        # Ignore non-string message content
+        # because we assume that the images/audio are already added to the message
+        if (message.images is not None and len(message.images) > 0) or (
+            message.audio is not None and len(message.audio) > 0
+        ):
+            # Ignore non-string message content
+            # because we assume that the images/audio are already added to the message
+            if isinstance(message.content, str):
+                message_dict["content"] = [{"type": "text", "text": message.content}]
+                if message.images is not None:
+                    message_dict["content"].extend(images_to_message(images=message.images))
 
-            if message.videos is not None:
-                logger.warning("Video input is currently unsupported.")
+                if message.audio is not None:
+                    message_dict["content"].extend(audio_to_message(audio=message.audio))
+
+        if message.audio_output is not None:
+            message_dict["content"] = None
+            message_dict["audio"] = {"id": message.audio_output.id}
+
+        if message.videos is not None:
+            logger.warning("Video input is currently unsupported.")
 
         # OpenAI expects the tool_calls to be None if empty, not an empty list
         if message.tool_calls is not None and len(message.tool_calls) == 0:
-            message.tool_calls = None
+            message_dict["tool_calls"] = None
 
-        message_dict = message.serialize_for_model()
-        message_dict["role"] = self.role_map[message_dict["role"]]
+        # Manually add the content field even if it is None
+        if message.content is None:
+            message_dict["content"] = None
 
         return message_dict
 
@@ -296,8 +320,14 @@ class OpenAIChat(Model):
             )
         except RateLimitError as e:
             logger.error(f"Rate limit error from OpenAI API: {e}")
+            error_message = e.response.json().get("error", {})
+            error_message = (
+                error_message.get("message", "Unknown model error")
+                if isinstance(error_message, dict)
+                else error_message
+            )
             raise ModelProviderError(
-                message=e.response.json().get("error", {}).get("message", "Unknown model error"),
+                message=error_message,
                 status_code=e.response.status_code,
                 model_name=self.name,
                 model_id=self.id,
@@ -307,8 +337,14 @@ class OpenAIChat(Model):
             raise ModelProviderError(message=str(e), model_name=self.name, model_id=self.id) from e
         except APIStatusError as e:
             logger.error(f"API status error from OpenAI API: {e}")
+            error_message = e.response.json().get("error", {})
+            error_message = (
+                error_message.get("message", "Unknown model error")
+                if isinstance(error_message, dict)
+                else error_message
+            )
             raise ModelProviderError(
-                message=e.response.json().get("error", {}).get("message", "Unknown model error"),
+                message=error_message,
                 status_code=e.response.status_code,
                 model_name=self.name,
                 model_id=self.id,
@@ -344,8 +380,14 @@ class OpenAIChat(Model):
             )
         except RateLimitError as e:
             logger.error(f"Rate limit error from OpenAI API: {e}")
+            error_message = e.response.json().get("error", {})
+            error_message = (
+                error_message.get("message", "Unknown model error")
+                if isinstance(error_message, dict)
+                else error_message
+            )
             raise ModelProviderError(
-                message=e.response.json().get("error", {}).get("message", "Unknown model error"),
+                message=error_message,
                 status_code=e.response.status_code,
                 model_name=self.name,
                 model_id=self.id,
@@ -355,8 +397,14 @@ class OpenAIChat(Model):
             raise ModelProviderError(message=str(e), model_name=self.name, model_id=self.id) from e
         except APIStatusError as e:
             logger.error(f"API status error from OpenAI API: {e}")
+            error_message = e.response.json().get("error", {})
+            error_message = (
+                error_message.get("message", "Unknown model error")
+                if isinstance(error_message, dict)
+                else error_message
+            )
             raise ModelProviderError(
-                message=e.response.json().get("error", {}).get("message", "Unknown model error"),
+                message=error_message,
                 status_code=e.response.status_code,
                 model_name=self.name,
                 model_id=self.id,
@@ -385,8 +433,14 @@ class OpenAIChat(Model):
             )  # type: ignore
         except RateLimitError as e:
             logger.error(f"Rate limit error from OpenAI API: {e}")
+            error_message = e.response.json().get("error", {})
+            error_message = (
+                error_message.get("message", "Unknown model error")
+                if isinstance(error_message, dict)
+                else error_message
+            )
             raise ModelProviderError(
-                message=e.response.json().get("error", {}).get("message", "Unknown model error"),
+                message=error_message,
                 status_code=e.response.status_code,
                 model_name=self.name,
                 model_id=self.id,
@@ -396,8 +450,14 @@ class OpenAIChat(Model):
             raise ModelProviderError(message=str(e), model_name=self.name, model_id=self.id) from e
         except APIStatusError as e:
             logger.error(f"API status error from OpenAI API: {e}")
+            error_message = e.response.json().get("error", {})
+            error_message = (
+                error_message.get("message", "Unknown model error")
+                if isinstance(error_message, dict)
+                else error_message
+            )
             raise ModelProviderError(
-                message=e.response.json().get("error", {}).get("message", "Unknown model error"),
+                message=error_message,
                 status_code=e.response.status_code,
                 model_name=self.name,
                 model_id=self.id,
@@ -428,8 +488,14 @@ class OpenAIChat(Model):
                 yield chunk
         except RateLimitError as e:
             logger.error(f"Rate limit error from OpenAI API: {e}")
+            error_message = e.response.json().get("error", {})
+            error_message = (
+                error_message.get("message", "Unknown model error")
+                if isinstance(error_message, dict)
+                else error_message
+            )
             raise ModelProviderError(
-                message=e.response.json().get("error", {}).get("message", "Unknown model error"),
+                message=error_message,
                 status_code=e.response.status_code,
                 model_name=self.name,
                 model_id=self.id,
@@ -439,8 +505,14 @@ class OpenAIChat(Model):
             raise ModelProviderError(message=str(e), model_name=self.name, model_id=self.id) from e
         except APIStatusError as e:
             logger.error(f"API status error from OpenAI API: {e}")
+            error_message = e.response.json().get("error", {})
+            error_message = (
+                error_message.get("message", "Unknown model error")
+                if isinstance(error_message, dict)
+                else error_message
+            )
             raise ModelProviderError(
-                message=e.response.json().get("error", {}).get("message", "Unknown model error"),
+                message=error_message,
                 status_code=e.response.status_code,
                 model_name=self.name,
                 model_id=self.id,
