@@ -26,7 +26,10 @@ class ModelResponse:
     tool_calls: List[Dict[str, Any]] = field(default_factory=list)
     event: str = ModelResponseEvent.assistant_response.value
 
+    provider_data: Optional[Dict[str, Any]] = None
+
     thinking: Optional[str] = None
+    redacted_thinking: Optional[str] = None
     reasoning_content: Optional[str] = None
 
     response_usage: Optional[Any] = None
