@@ -1,6 +1,7 @@
 import json
 
 from agno.tools import Toolkit
+from agno.utils.log import logger
 
 try:
     import yfinance as yf
@@ -54,6 +55,7 @@ class YFinanceTools(Toolkit):
             str: The current stock price or error message.
         """
         try:
+            logger.debug(f"Fetching current price for {symbol}")
             stock = yf.Ticker(symbol)
             # Use "regularMarketPrice" for regular market hours, or "currentPrice" for pre/post market
             current_price = stock.info.get("regularMarketPrice", stock.info.get("currentPrice"))
@@ -75,6 +77,8 @@ class YFinanceTools(Toolkit):
             if company_info_full is None:
                 return f"Could not fetch company info for {symbol}"
 
+            logger.debug(f"Fetching company info for {symbol}")
+
             company_info_cleaned = {
                 "Name": company_info_full.get("shortName"),
                 "Symbol": company_info_full.get("symbol"),
@@ -125,6 +129,7 @@ class YFinanceTools(Toolkit):
           str: The current stock price or error message.
         """
         try:
+            logger.debug(f"Fetching historical prices for {symbol}")
             stock = yf.Ticker(symbol)
             historical_price = stock.history(period=period, interval=interval)
             return historical_price.to_json(orient="index")
@@ -154,6 +159,7 @@ class YFinanceTools(Toolkit):
                     - '52_week_low': The 52-week low price of the stock.
         """
         try:
+            logger.debug(f"Fetching fundamentals for {symbol}")
             stock = yf.Ticker(symbol)
             info = stock.info
             fundamentals = {
@@ -184,6 +190,7 @@ class YFinanceTools(Toolkit):
             dict: JSON containing income statements or an empty dictionary.
         """
         try:
+            logger.debug(f"Fetching income statements for {symbol}")
             stock = yf.Ticker(symbol)
             financials = stock.financials
             return financials.to_json(orient="index")
@@ -200,6 +207,7 @@ class YFinanceTools(Toolkit):
             dict: JSON containing key financial ratios.
         """
         try:
+            logger.debug(f"Fetching key financial ratios for {symbol}")
             stock = yf.Ticker(symbol)
             key_ratios = stock.info
             return json.dumps(key_ratios, indent=2)
@@ -216,6 +224,7 @@ class YFinanceTools(Toolkit):
             str: JSON containing analyst recommendations.
         """
         try:
+            logger.debug(f"Fetching analyst recommendations for {symbol}")
             stock = yf.Ticker(symbol)
             recommendations = stock.recommendations
             return recommendations.to_json(orient="index")
@@ -233,6 +242,7 @@ class YFinanceTools(Toolkit):
             str: JSON containing company news and press releases.
         """
         try:
+            logger.debug(f"Fetching company news for {symbol}")
             news = yf.Ticker(symbol).news
             return json.dumps(news[:num_stories], indent=2)
         except Exception as e:
@@ -250,6 +260,7 @@ class YFinanceTools(Toolkit):
             str: JSON containing technical indicators.
         """
         try:
+            logger.debug(f"Fetching technical indicators for {symbol}")
             indicators = yf.Ticker(symbol).history(period=period)
             return indicators.to_json(orient="index")
         except Exception as e:
